/**
    (C) Copyright 2011 Dominik Wagenfuehr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this program. If not, see
    <http://www.gnu.org/licenses/>.
*/

#include "botdata.hh"
#include <vector>

// Konstruktor.
BotData::BotData()
: mNumMaxRounds(0), mRound(0), mPoints(0), mOpponentPoints(0), my_last_offer(0), opponent_last_offer(0), opponent_last_accepted(true), opponent_last2_rejected(false), i_accepted(), opponent_accepted()
{
}

// Setzt alle Daten zurueck.
void BotData::reset()
{
    mNumMaxRounds = 0;
    mRound = 0;
    mPoints = 0;
    mOpponentPoints = 0;
    my_last_offer = 0;
    opponent_last_offer = 0;
    opponent_last_accepted = true;
    opponent_last2_rejected = false;
    //i_accepted = std::vector<int>;
    //opponent_accepted = std::vector<int>;
}

// Speichere Anzahl an Runden.
void BotData::setRounds( const int value )
{
    mNumMaxRounds = value;
}

// Eine neue Runde wird gestartet.
void BotData::startNewRound()
{
    // Rundenzaehler erhoehen.
    mRound++;
}

// Fuegt erreichte Punktzahl hinzu.
void BotData::addPoints( const int points )
{
    mPoints += points;
}

// Fuegt erreichte Punktzahl des Gegners hinzu.
void BotData::addOpponentPoints( const int points )
{
    mOpponentPoints += points;
}

void BotData::setMyLastOffer(const int offer) {
	my_last_offer = offer;
}

void BotData::setOpponentLastOffer(const int offer) {
	opponent_last_offer = offer;
}

void BotData::setOpponentLastAccepted(const bool accepted) {
	opponent_last_accepted = accepted;
}

void BotData::setOpponentLast2Rejected(const bool rejected) {
	opponent_last2_rejected = rejected;
}

void BotData::addMyAccept(const int offer) {
	i_accepted.push_back(offer);
}

void BotData::addOpponentAccept(const int offer) {
	opponent_accepted.push_back(offer);
}

int BotData::getMyAcceptAvg() const {
	//avoid division by 0
	if(i_accepted.size() == 0) {
		return 0;
	}
	int sum = 0;
	for(unsigned int i=0; i<i_accepted.size(); ++i) {
		sum += i_accepted[i];
	}
	return (int) (sum / i_accepted.size());
}

int BotData::getOpponentAcceptAvg() const {
	//avoid division by 0
	if(opponent_accepted.size() == 0) {
		return 0;
	}
	int sum = 0;
	for(unsigned int i=0; i<opponent_accepted.size(); ++i) {
		sum += opponent_accepted[i];
	}
	return (int) (sum / opponent_accepted.size());
}
